
require 'sketchup.rb'

module DM::BoolTools

  class BoolSelfUnionTool

    include DM::BoolTools

    include DM::BoolTools::ToolMixin



    def initialize

      icon = File.join(PATH, "Icons", "self_union_cursor.png")

      @cursor = UI.create_cursor(icon, 12, 12)

      @bool_operation_string = "Single Object Union"

      icon = File.join(PATH, "Icons", "wait.png")

      @wait_cursor = UI.create_cursor(icon, 12, 12)

    end



    def perform_boolean(instance)

      UI.set_cursor(@wait_cursor)

      box = get_bounds_of_instances([instance])

      previewer = BTPreviewer.new

      previewer.set_bounds(box)

      begin

        start_operation(@bool_operation_string)

        changed_instances = Boolean.self_union(instance).changed_instances

        #todo set attributes on the instances definition

        set_bool_tools_modified_attribute(changed_instances)

      rescue Exception => e

        p "BoolTools Operation Error:"

        p e.message

        p e.backtrace

      ensure

        lic_status = Licensing::License.new.get_state

        if lic_status == Licensing::LICENSED

          Sketchup.active_model.commit_operation

        else

          changed_instances.each { |inst| previewer.add_preview_mesh(inst) }

          Sketchup.active_model.abort_operation

          Sketchup.active_model.select_tool(previewer)

        end

      end

    end



    def activate

      reset

    end



    def onSetCursor

      UI.set_cursor(@cursor) if (@cursor)

    end



    def deactivate(view)

      @first = nil

      @state = 0

      @sel.clear

      Sketchup.set_status_text

    end



    def onLButtonDown(flags, x, y, view)

      if (@state == 0)  #we are getting the only operand

        ph = view.pick_helper

        ph.do_pick(x, y)

        best = ph.best_picked

        if ((best.class == Sketchup::Group) || (best.class == Sketchup::ComponentInstance))

          @sel.add(best)

          @first = best

          result = validate_operands(@first)

          perform_boolean(@first) if (result)

          self.reset

        end

      end

    end  #end onLbuttondown



    def onMouseMove(flags, x, y, view)

      ph = view.pick_helper

      ph.do_pick(x, y)

      best = ph.best_picked

      if ((best.class == Sketchup::Group) || (best.class == Sketchup::ComponentInstance))

        @sel.clear

        @sel.add(best)

      else

        @sel.clear

      end

    end



    def reset

      @first = nil  #the only operand

      @state = 0

      @sel = Sketchup.active_model.selection

      @sel.clear

      Sketchup.set_status_text("Click a group or component that contains nested solids")

    end



  end  #end class

end